/*
 * (C) Copyright 2001
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/*
 * Date & Time support for Philips PCF8563 RTC
 */
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "stm32f0xx_conf.h"

#include "rtc.h"
#include "date.h"

#define FEBRUARY		2
#define	STARTOFTIME		1970
#define SECDAY			86400L
#define SECYR			(SECDAY * 365)
#define	leapyear(year)		((year) % 4 == 0)
#define	days_in_year(a) 	(leapyear(a) ? 366 : 365)
#define	days_in_month(a) 	(month_days[(a) - 1])

static int month_days[12] = {
	31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31
};

/*
 * This only works for the Gregorian calendar - i.e. after 1752 (in the UK)
 */
void GregorianDay(struct rtc_time * tm)
{
	int leapsToDate;
	int lastYear;
	int day;
	int MonthOffset[] = { 0,31,59,90,120,151,181,212,243,273,304,334 };

	lastYear=tm->tm_year-1;

	/*
	 * Number of leap corrections to apply up to end of last year
	 */
	leapsToDate = lastYear/4 - lastYear/100 + lastYear/400;

	/*
	 * This year is a leap year if it is divisible by 4 except when it is
	 * divisible by 100 unless it is divisible by 400
	 *
	 * e.g. 1904 was a leap year, 1900 was not, 1996 is, and 2000 will be
	 */
	if((tm->tm_year%4==0) &&
	   ((tm->tm_year%100!=0) || (tm->tm_year%400==0)) &&
	   (tm->tm_mon>2)) {
		/*
		 * We are past Feb. 29 in a leap year
		 */
		day=1;
	} else {
		day=0;
	}

	day += lastYear*365 + leapsToDate + MonthOffset[tm->tm_mon-1] + tm->tm_mday;

	tm->tm_wday=day%7;
}

void to_tm(int tim, struct rtc_time * tm)
{
	register int    i;
	register long   hms, day;

	day = tim / SECDAY;
	hms = tim % SECDAY;

	/* Hours, minutes, seconds are easy */
	tm->tm_hour = hms / 3600;
	tm->tm_min = (hms % 3600) / 60;
	tm->tm_sec = (hms % 3600) % 60;

	/* Number of years in days */
	for (i = STARTOFTIME; day >= days_in_year(i); i++) {
		day -= days_in_year(i);
	}
	tm->tm_year = i;

	/* Number of months in days left */
	if (leapyear(tm->tm_year)) {
		days_in_month(FEBRUARY) = 29;
	}
	for (i = 1; day >= days_in_month(i); i++) {
		day -= days_in_month(i);
	}
	days_in_month(FEBRUARY) = 28;
	tm->tm_mon = i;

	/* Days are what is left over (+1) from all that. */
	tm->tm_mday = day + 1;

	/*
	 * Determine the day of week
	 */
	GregorianDay(tm);
}



/* Converts Gregorian date to seconds since 1970-01-01 00:00:00.
 * Assumes input in normal date format, i.e. 1980-12-31 23:59:59
 * => year=1980, mon=12, day=31, hour=23, min=59, sec=59.
 *
 * [For the Julian calendar (which was used in Russia before 1917,
 * Britain & colonies before 1752, anywhere else before 1582,
 * and is still in use by some communities) leave out the
 * -year/100+year/400 terms, and add 10.]
 *
 * This algorithm was first published by Gauss (I think).
 *
 * WARNING: this function will overflow on 2106-02-07 06:28:16 on
 * machines were long is 32-bit! (However, as time_t is signed, we
 * will already get problems at other places on 2038-01-19 03:14:08)
 */
unsigned long
maketime (unsigned int year, unsigned int mon,
	unsigned int day, unsigned int hour,
	unsigned int min, unsigned int sec)
{
	if (0 >= (int) (mon -= 2)) {	/* 1..12 -> 11,12,1..10 */
		mon += 12;		/* Puts Feb last since it has leap day */
		year -= 1;
	}

	return (((
		(unsigned long) (year/4 - year/100 + year/400 + 367*mon/12 + day) +
			year*365 - 719499
	    )*24 + hour /* now have hours */
	  )*60 + min /* now have minutes */
	)*60 + sec; /* finally seconds */
}

/*-----------------------------------------------------------------------------*/
/* based on newlib implementation */
/*-----------------------------------------------------------------------------*/

//#define YEAR_BASE 1900
//#define EPOCH_YEAR 2000
//#define EPOCH_WDAY 6

#define SECSPERHOUR (60 * 60)
#define SECSPERDAY (SECSPERHOUR * 24)
#define ISLEAP(y) ((((y) % 4) == 0 && ((y) % 100) != 0) || ((y) % 400) == 0)
#define YEAR_LENGTH(leap) ((leap) ? 366 : 365)
#define MON_LENGTH(leap, mon) (month_days[mon] + (((mon) == 1) && (leap)))

struct rtc_time *rtc_to_time(uint32_t rtcval, struct rtc_time *res)
{
   /* base decision about std/dst time on current time */
   long days = ((int32_t)rtcval) / SECSPERDAY;
   long rem = ((int32_t)rtcval) % SECSPERDAY;

   while(rem < 0)
   {
      rem += SECSPERDAY;
      days--;
   }

   while(rem >= SECSPERDAY)
   {
      rem -= SECSPERDAY;
      days++;
   }

   /* compute hour, min, and sec */
   res->tm_hour = (int)(rem / SECSPERHOUR);
   rem %= SECSPERHOUR;
   res->tm_min = (int)(rem / 60);
   res->tm_sec = (int)(rem % 60);

   /* compute day of week */
   if((res->tm_wday = ((EPOCH_WDAY + days) % 7)) < 0) res->tm_wday += 7;

   /* compute year & day of year */
   int y = EPOCH_YEAR;
   int yleap;
   if(days >= 0)
   {
      while(1)
      {
         yleap = ISLEAP(y);
         if(days < YEAR_LENGTH(yleap)) break;
         y++;
         days -= YEAR_LENGTH(yleap);
      }
   }
   else
   {
      do
      {
         y--;
         yleap = ISLEAP(y);
         days += YEAR_LENGTH(yleap);
      }
      while (days < 0);
   }

   res->tm_year = y - YEAR_BASE;
   res->tm_yday = days;

   res->tm_mon = 0;
   while(days >= MON_LENGTH(yleap, res->tm_mon))
   {
      days -= MON_LENGTH(yleap, res->tm_mon);
      res->tm_mon++;
   }
   res->tm_mday = days + 1;

   res->tm_isdst = 0;

   return res;
}

static const int days_before_month[12] =
{0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334};

uint32_t rtc_from_time(const struct rtc_time *timp)
{
   int year_abs = timp->tm_year + YEAR_BASE;

   /* compute hours, minutes, seconds */
   int32_t tim = timp->tm_sec + (timp->tm_min * 60) + (timp->tm_hour * SECSPERHOUR);

   /* compute days in year */
   long days = timp->tm_mday - 1 + days_before_month[timp->tm_mon];
   if(timp->tm_mon > 1 && ISLEAP(year_abs)) days++;

   /* compute days in other years */
   int year = year_abs;
   if(year > EPOCH_YEAR)
   {
      for(year = EPOCH_YEAR; year < year_abs; year++) days += YEAR_LENGTH(ISLEAP(year));
   }
   else if (year < EPOCH_YEAR)
   {
      for(year = EPOCH_YEAR - 1; year >= year_abs; year--) days -= YEAR_LENGTH(ISLEAP(year));
   }

   /* compute total seconds */
   tim += (days * SECSPERDAY);

   return (uint32_t)tim;
}

int time_to_str(char *buf, size_t sz, const struct rtc_time *tim)
{
   return sniprintf(buf, sz, "%02d:%02d:%02d %02d-%02d-%d",
         tim->tm_hour, tim->tm_min, tim->tm_sec,
         tim->tm_mday, tim->tm_mon + 1, YEAR_BASE + tim->tm_year);
}

int validate_time(const struct rtc_time *tim)
{
   return tim->tm_hour >= 0 && tim->tm_hour <= 23 &&
         tim->tm_min >= 0 && tim->tm_min <= 59 &&
         tim->tm_sec >= 0 && tim->tm_sec <= 59;
}

int validate_date(const struct rtc_time *tim)
{
   return tim->tm_mday >= 1 && tim->tm_mday <= 31 &&
         tim->tm_mon >= 0 && tim->tm_mon <= 11;
}

int parse_time(const char *str, struct rtc_time *tim)
{
   char *end;
   /* hours */
   tim->tm_hour = strtol(str, &end, 10);
   if(end != str && *end)
   {
      str = end + 1;
      /* minutes */
      tim->tm_min = strtol(str, &end, 10);
      if(end != str)
      {
         /* seconds (optional) */
         tim->tm_sec = *end ? strtol(end + 1, NULL, 10) : 0;

         return validate_time(tim);
      }
   }
   return 0;
}

int parse_date(const char *str, struct rtc_time *tim)
{
   char *end;
   /* day */
   tim->tm_mday = strtol(str, &end, 10);
   if(end != str && *end)
   {
      str = end + 1;
      /* month */
      tim->tm_mon = strtol(str, &end, 10);
      if(end != str && *end)
      {
         str = end + 1;
         /* year */
         tim->tm_year = strtol(str, &end, 10);
         if(end != str)
         {
            tim->tm_mon--;
            tim->tm_year -= YEAR_BASE;

            return validate_date(tim);
         }
      }
   }
   return 0;
}
