

#ifndef _LCD_H
#define _LCD_H

#define LCD_ADDR     (0x27 << 1)
#define LCD_ROW      2 // 1602 type, 2 line x 16


//Commands
#define LCD_CLEARDISPLAY            0x01
#define LCD_RETURNHOME              0x02
#define LCD_ENTRYMODESET            0x04
#define LCD_DISPLAYCONTROL          0x08
#define LCD_CURSORSHIFT             0x10
#define LCD_FUNCTIONSET             0x20
#define LCD_SETCGRAMADDR            0x40
#define LCD_SETDDRAMADDR            0x80

//Flags for display entry mode
#define LCD_ENTRYRIGHT              0x00
#define LCD_ENTRYLEFT               0x02
#define LCD_ENTRYSHIFTINCREMENT     0x01
#define LCD_ENTRYSHIFTDECREMENT     0x00

//Flags for display on/off control
#define LCD_DISPLAYON               0x04
#define LCD_CURSORON                0x02
#define LCD_BLINKON                 0x01

//Flags for display/cursor shift
#define LCD_DISPLAYMOVE             0x08
#define LCD_CURSORMOVE              0x00
#define LCD_MOVERIGHT               0x04
#define LCD_MOVELEFT                0x00

//Flags for function set
#define LCD_8BITMODE                0x10
#define LCD_4BITMODE                0x00
#define LCD_2LINE                   0x08
#define LCD_1LINE                   0x00
#define LCD_5x10DOTS                0x04
#define LCD_5x8DOTS                 0x00


void LCD_Init(uint8_t lcd_addr);
void LCD_HomeCursor(void);
void LCD_ClearDisplay(void);
void LCD_setCursor(uint8_t col, uint8_t row);
void LCD_PrintStr(unsigned char *str);
void LCD_ScrollLeft(void);
void LCD_ScrollRight(void);
void LCD_SetDDRAMAddr(unsigned char addr);
void LCD_GotoLine2(void);
void LCD_EntryModeDispShift(unsigned char on);
void LCD_NoBlink(void);
void LCD_Blink(void);

void LCD_PWMBacklight(uint8_t pwm); // write to STM32F030 I2C Slave that controls LCD backlight..

#endif
