/************************************************************************************
 *  File:     main.c
 *  Purpose:  Cortex-M0/0+/1 main file.
 *            Replace with your code.
 *  Date:     05 July 2013
 *  Info:     If __NO_SYSTEM_INIT is defined in the Build options,
 *            the startup code will not branch to SystemInit()
 *            and the function can be removed
 ************************************************************************************/

#include <stdint.h>
#include <stdio.h>
#include <string.h>

#include "stm32f0xx_conf.h"

#include "graham-pt.h"
#include "main.h"
#include "ringbuffer.h"

#include "rtc.h"
#include "date.h"
#include "pcf8563.h"
#include "menu.h"
#include "lcd.h"
#include "delay.h"
#include "dbgprintf.h"

static void InitGPIO(void);
static void EXTI4_Config(void);
static void Timer14_Config(void);

static int TimeDisp_PThread(struct pt *pt);
static int NECIR_DecodePThread(struct pt *pt);

uint32_t irraw_buffer[16];;
ringbuffer_t *pRingbuffer;

struct rtc_time rtcTime;


void setdefaultTime(void)
{
   char *curTime = "12:30:00";
   char *curDate = "13-07-2018";  // 1 Jul 2018
   struct rtc_time tim;
   uint32_t unix_time;

   parse_date(curDate, &tim);
   parse_time(curTime, &tim);

   // convert to unix time
   unix_time = rtc_from_time(&tim);

   // then back to rtcTime struct so that the wday is properly set from tim.
   rtc_to_time(unix_time, &tim);

   pcf8563_set(&tim);
   DelayMSec(500);
}

/*********************************************************************
 *
 *  main()
 *
 *********************************************************************/
int main(void)
{
   /******************************************************************
    *
    *  Place your code here.
    ******************************************************************/
   u32 starttime;
   static struct pt pt1;
   static struct pt timedisp_pt;
   int i;

   SystemCoreClockUpdate();

   if (SysTick_Config(SystemCoreClock / 1000)) // 1ms interval
   {
      /* Capture error */
      while (1);
   }

#ifdef SEGGER_RTT
   SEGGER_RTT_ConfigUpBuffer(0, NULL, NULL, 0, SEGGER_RTT_MODE_BLOCK_IF_FIFO_FULL); //SEGGER_RTT_MODE_NO_BLOCK_SKIP);

   SEGGER_RTT_WriteString(0, "SEGGER RTT - irDecoder\r\n\r\n");
#endif

   RCC_APB2PeriphClockCmd(RCC_APB2Periph_SYSCFG, ENABLE);
   RCC_APB2PeriphClockCmd(RCC_APB2Periph_DBGMCU, ENABLE);
   DBGMCU_APB1PeriphConfig(DBGMCU_TIM3_STOP, ENABLE);

   pRingbuffer = (ringbuffer_t *)irraw_buffer;
   ringbuffer_init(pRingbuffer, sizeof(irraw_buffer));

   I2C_Recovery();

   InitGPIO();
   Timer14_Config();

   LCD_Init(LCD_ADDR);

   if (pcf8563_checkClockIntegrity())
   {
      pcf8563_reset();
      setdefaultTime();
   }

   PT_INIT(&pt1);
   PT_INIT(&timedisp_pt);
   menu_init();

   LCD_PWMBacklight(200);

   while(1)
   {
      TimeDisp_PThread(&timedisp_pt);
      menu_loop();
      NECIR_DecodePThread(&pt1);
   }

   return 0;
}


static void InitGPIO(void)
{
   GPIO_InitTypeDef GPIO_InitStructure;

   /* GPIOA Clocks enable */
   RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);

   /* GPIOA Configuration: PA5 output push-pull */
   GPIO_InitStructure.GPIO_Pin =  GPIO_Pin_5;
   GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
   GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
   GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
   GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP ;
   GPIO_Init(GPIOA, &GPIO_InitStructure);

   cbit(GPIOA, 5);
}


/**
 * @brief  Configure PA4 in interrupt mode
 * @param  None
 * @retval None
 */
static void EXTI4_Config(void)
{
   EXTI_InitTypeDef   EXTI_InitStructure;
   GPIO_InitTypeDef   GPIO_InitStructure;
   NVIC_InitTypeDef   NVIC_InitStructure;

   /* Enable GPIOA clock */
   RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);

   /* Configure PA4 pin as input floating */
   GPIO_InitStructure.GPIO_Pin = GPIO_Pin_4;
   GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
   GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_DOWN;
   GPIO_Init(GPIOA, &GPIO_InitStructure);

   /* Enable SYSCFG clock */
   RCC_APB2PeriphClockCmd(RCC_APB2Periph_SYSCFG, ENABLE);
   /* Connect EXTI4 Line to PA4 pin */
   SYSCFG_EXTILineConfig(EXTI_PortSourceGPIOA, EXTI_PinSource4);

   /* Configure EXTI4 line */
   EXTI_InitStructure.EXTI_Line = EXTI_Line4;
   EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
   EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising;
   EXTI_InitStructure.EXTI_LineCmd = ENABLE;
   EXTI_Init(&EXTI_InitStructure);

   /* Enable and set EXTI4 Interrupt */
   NVIC_InitStructure.NVIC_IRQChannel = EXTI4_15_IRQn;
   NVIC_InitStructure.NVIC_IRQChannelPriority = 0x00;
   NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
   NVIC_Init(&NVIC_InitStructure);
}



// Configure Timer 14 with PA4 as input capture.
static void Timer14_Config(void)
{
   GPIO_InitTypeDef GPIO_InitStructure;
   TIM_TimeBaseInitTypeDef TIM_TimeBase_InitStructure;
   TIM_ICInitTypeDef  TIM_ICInitStructure;
   NVIC_InitTypeDef NVIC_InitStructure;

   /* GPIOA, Timer14 Clocks enable */
   RCC_AHBPeriphClockCmd(RCC_AHBPeriph_GPIOA, ENABLE);
   RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM14, ENABLE);

   /* GPIOA Configuration: PA4 AF */
   GPIO_InitStructure.GPIO_Pin =  GPIO_Pin_4;
   GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
   GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
   GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
   GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL ;
   GPIO_Init(GPIOA, &GPIO_InitStructure);

   GPIO_PinAFConfig(GPIOA, GPIO_PinSource4, GPIO_AF_4); // PA4


   // Configure Timer 14
   TIM_TimeBase_InitStructure.TIM_ClockDivision  = TIM_CKD_DIV1;
   TIM_TimeBase_InitStructure.TIM_CounterMode = TIM_CounterMode_Up;
   TIM_TimeBase_InitStructure.TIM_Period = 65535 ;
   TIM_TimeBase_InitStructure.TIM_Prescaler = 12 - 1 ;          //Vorteiler 48/12 => 4MHz
   TIM_TimeBase_InitStructure.TIM_RepetitionCounter = 0 ;

   TIM_TimeBaseInit(TIM14, &TIM_TimeBase_InitStructure);

   /* TIM1 configuration: Input Capture mode ---------------------
      The external signal is connected to TIM14 CH1 pin (PA4)
      The Rising edge is used as active edge,
      The TIM14 CCR1 is used to compute the frequency value
   ------------------------------------------------------------ */
   TIM_ICInitStructure.TIM_Channel = TIM_Channel_1;
   TIM_ICInitStructure.TIM_ICPolarity = TIM_ICPolarity_BothEdge;
   TIM_ICInitStructure.TIM_ICSelection = TIM_ICSelection_DirectTI;
   TIM_ICInitStructure.TIM_ICPrescaler = TIM_ICPSC_DIV1; // Capture performed each time an edge is detected on the capture input
   TIM_ICInitStructure.TIM_ICFilter = 0x0;

   TIM_ICInit(TIM14, &TIM_ICInitStructure);

   /* Enable the CC1 Interrupt Request */
   TIM_ITConfig(TIM14, TIM_IT_CC1, ENABLE);

   /* Enable the TIM14 global Interrupt */
   NVIC_InitStructure.NVIC_IRQChannel = TIM14_IRQn;
   NVIC_InitStructure.NVIC_IRQChannelPriority = 0;
   NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
   NVIC_Init(&NVIC_InitStructure);

   TIM_Cmd(TIM14, ENABLE);
}



// empirically
#define NEC_AGC_PULSE      9050 // in us..
#define NEC_AGC_SPACE      4500
#define NEC_SHORT_PULSE    570
#define NEC_LONG_PULSE     1670
#define NEC_REPEAT_PULSE   2200

static uint16_t necAGCSpace = NEC_AGC_SPACE;
static uint16_t necShortPulse = NEC_SHORT_PULSE;
static uint16_t necLongPulse = NEC_LONG_PULSE;
static uint16_t necRepeatPulse = NEC_REPEAT_PULSE;

static int NECIR_DecodePThread(struct pt *pt)
{
   PT_YIELDING();
   uint8_t rd;
   uint16_t diff;
   uint16_t tolerance;
   static uint8_t isRepeat;
   static uint16_t i;
   static uint16_t j;
   static uint8_t remote_code[4];

   if (ringbuffer_isempty(pRingbuffer))
   {
      return 0;
   }

   rd = ringbuffer_read(pRingbuffer, &diff, sizeof(diff));
   if (rd == sizeof(diff))
   {
      diff = diff >> 2; // div 4, since each timer clock is 0.25us (4Mhz)..
   }
   else
   {
      return 0;
   }

   /* A protothread function must begin with PT_BEGIN() which takes a
      pointer to a struct pt. */
   PT_BEGIN(pt);

   /* We loop forever here. */
   while(1)
   {
      isRepeat = 0;

      // Leader pulse
      PT_WAIT_UNTIL(pt, abs(diff - NEC_AGC_PULSE) <= 200);
      PT_YIELD(pt);

      // Leader space OR Repeat Space
      PT_WAIT_UNTIL(pt, (abs(diff - necAGCSpace) <= 200) || (abs(diff - necRepeatPulse) <= 200));
      if (abs(diff - necAGCSpace) <= 200)
      {
         isRepeat = 0;
      }
      else
      {
         isRepeat = 1;
      }
      PT_YIELD(pt);

      if (!isRepeat)
      {
         memset(remote_code, 0, sizeof(remote_code));
         j = 0;

         for(; j<4; j++)
         {
            // 8 bits, LSB first
            i = 0;
            for(; i<8; i++)
            {
               // first is short pulse
               if (abs(diff - necShortPulse) <= 200)
               {
                  PT_YIELD(pt);
               }
               else
               {
                  PT_RESTART(pt);
               }

               // next capture interval will determine whether the bit is 0 (short pulse) or 1 (Long pulse).
               tolerance = abs(diff - necShortPulse);
               remote_code[j] >>= 1;
               if (tolerance <= 200)
               {
                  remote_code[j] |= 0;
               }
               else
               {
                  tolerance = abs(diff - necLongPulse);
                  if (tolerance <= 300)
                  {
                     remote_code[j] |= 0x80L;
                  }
                  else
                  {
                     PT_RESTART(pt);
                  }
               }
               PT_YIELD(pt);
            }
         }
      }

      PT_WAIT_UNTIL(pt, abs(diff - necShortPulse) <= 200); // NEC remote protocol - there is a final pulse of 562us.

      if (isRepeat)
      {
      }
      else
      {
         // decode is complete
         //dbgprintf(("%02x, %02x, %02x, %02x\n", remote_code[0], remote_code[1], remote_code[2], remote_code[3]));
         menu_keymap(remote_code);
      }

      /* And we loop. */
   }

   /* All protothread functions must end with PT_END() which takes a
      pointer to a struct pt. */
   PT_END(pt);
}


static int TimeDisp_PThread(struct pt *pt)
{
   //PT_YIELDING();
   static u32 starttime;

   struct rtc_time *tim;
   char strbuf[20];

   /* A protothread function must begin with PT_BEGIN() which takes a
      pointer to a struct pt. */
   PT_BEGIN(pt);
   starttime = syscnt;

   /* We loop forever here. */
   while(1)
   {
      PT_WAIT_UNTIL(pt, (IsDelayOver(starttime, 500) > 0));
      starttime = syscnt;

      if ((menu_GetCmdMode() == MODE_MENU_IDLE))
      {
         LCD_setCursor(0, 0);
         pcf8563_get(&rtcTime);
         tim = &rtcTime;
         sniprintf(strbuf, sizeof(strbuf), "%02d:%02d:%02d",
               tim->tm_hour, tim->tm_min, tim->tm_sec);
         LCD_PrintStr((uint8_t *)strbuf);
         LCD_setCursor(0, 1);
         sniprintf(strbuf, sizeof(strbuf), "%02d-%02d-%d",
               tim->tm_mday, tim->tm_mon + 1, YEAR_BASE + tim->tm_year);
         LCD_PrintStr((uint8_t *)strbuf);
      }
   }

   /* All protothread functions must end with PT_END() which takes a
      pointer to a struct pt. */
   PT_END(pt);
}


