/*
 * (C) Copyright 2001
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/*
 * Date & Time support for Philips PCF8563 RTC
 */

/* #define	DEBUG	*/
#include "stm32f0xx_conf.h"
#include <string.h>

#include "rtc.h"
#include "lib_i2c.h"
#include "pcf8563.h"
#include "date.h"


typedef unsigned char uchar;

#define PCF8563_ADDR_WRITE 0xA2  //Write address  R/W =0
#define PCF8563_ADDR_READ  0xA3  //Read address R/W = 1

static uchar pcf8563_read  (uchar reg);
static void  pcf8563_write (uchar reg, uchar val);
static int pcf8563_readbuf(uchar reg, uchar *buf, uchar num);
static int pcf8563_writebuf(uchar reg, uchar *buf, uchar num);
static uchar bin2bcd   (unsigned int n);
static unsigned bcd2bin(uchar c);


static uchar rtcBuffer[12];


/* ------------------------------------------------------------------------- */

void pcf8563_get (struct rtc_time *tmp)
{
   uchar sec, min, hour, mday, wday, mon_cent, year;
   uchar buf[12];

   if (pcf8563_readbuf(VL_seconds_reg, buf, 11) == 11)  // read from 0x02 - 0x0C
   {
      sec   = buf[0];
      min   = buf[1];
      hour  = buf[2];
      mday  = buf[3];
      wday  = buf[4];
      mon_cent= buf[5];
      year  = buf[6];

      //	   debug ( "Get RTC year: %02x mon/cent: %02x mday: %02x wday: %02x "
      //	      "hr: %02x min: %02x sec: %02x\n",
      //	      year, mon_cent, mday, wday,
      //	      hour, min, sec );
      //	   debug ( "Alarms: wday: %02x day: %02x hour: %02x min: %02x\n",
      //	         buf[7],
      //	         buf[8],
      //	         buf[9],
      //	         buf[10] );

      if (sec & 0x80) {
         //	      puts ("### Warning: RTC Low Voltage - date/time not reliable\n");
      }

      tmp->tm_sec  = bcd2bin (sec  & 0x7F);
      tmp->tm_min  = bcd2bin (min  & 0x7F);
      tmp->tm_hour = bcd2bin (hour & 0x3F);
      tmp->tm_mday = bcd2bin (mday & 0x3F);
      tmp->tm_mon  = bcd2bin (mon_cent & 0x1F);
      tmp->tm_year = bcd2bin (year) + ((mon_cent & 0x80) ? 2000 : 1900);
      tmp->tm_year -= YEAR_BASE;
      tmp->tm_wday = bcd2bin (wday & 0x07);
      tmp->tm_yday = 0;
      tmp->tm_isdst= 0;

      //	   debug ( "Get DATE: %4d-%02d-%02d (wday=%d)  TIME: %2d:%02d:%02d\n",
      //	      tmp->tm_year, tmp->tm_mon, tmp->tm_mday, tmp->tm_wday,
      //	      tmp->tm_hour, tmp->tm_min, tmp->tm_sec);
   }
}

void pcf8563_set (struct rtc_time *tmp)
{
   uchar century;
   uchar buf[7];

   //	debug ( "Set DATE: %4d-%02d-%02d (wday=%d)  TIME: %2d:%02d:%02d\n",
   //		tmp->tm_year, tmp->tm_mon, tmp->tm_mday, tmp->tm_wday,
   //		tmp->tm_hour, tmp->tm_min, tmp->tm_sec);

   //	pcf8563_write (0x08, bin2bcd(tmp->tm_year % 100));
   //
   //	century = (tmp->tm_year >= 2000) ? 0x80 : 0;
   //	pcf8563_write (0x07, bin2bcd(tmp->tm_mon) | century);
   //
   //	pcf8563_write (0x06, bin2bcd(tmp->tm_wday));
   //	pcf8563_write (0x05, bin2bcd(tmp->tm_mday));
   //	pcf8563_write (0x04, bin2bcd(tmp->tm_hour));
   //	pcf8563_write (0x03, bin2bcd(tmp->tm_min ));
   //	pcf8563_write (0x02, bin2bcd(tmp->tm_sec ));

   century = ((YEAR_BASE + tmp->tm_year) >= 2000) ? 0x80 : 0;
   buf[0] = bin2bcd(tmp->tm_sec );
   buf[1] = bin2bcd(tmp->tm_min );
   buf[2] = bin2bcd(tmp->tm_hour);
   buf[3] = bin2bcd(tmp->tm_mday);
   buf[4] = bin2bcd(tmp->tm_wday);
   buf[5] = bin2bcd(tmp->tm_mon) | century;
   buf[6] = bin2bcd(tmp->tm_year % 100);

   pcf8563_writebuf(VL_seconds_reg, buf, 7);
}


void pcf8563_settime (struct rtc_time *tmp)
{
   uchar buf[3];

   buf[0] = bin2bcd(tmp->tm_sec );
   buf[1] = bin2bcd(tmp->tm_min );
   buf[2] = bin2bcd(tmp->tm_hour);

   pcf8563_writebuf(VL_seconds_reg, buf, 3);
}


void pcf8563_setdate (struct rtc_time *tmp)
{
   uchar century;
   uchar buf[4];

   // debug ( "Set DATE: %4d-%02d-%02d (wday=%d)  TIME: %2d:%02d:%02d\n",
   //    tmp->tm_year, tmp->tm_mon, tmp->tm_mday, tmp->tm_wday,
   //    tmp->tm_hour, tmp->tm_min, tmp->tm_sec);

   // pcf8563_write (0x08, bin2bcd(tmp->tm_year % 100));
   //
   // century = (tmp->tm_year >= 2000) ? 0x80 : 0;
   // pcf8563_write (0x07, bin2bcd(tmp->tm_mon) | century);
   //
   // pcf8563_write (0x06, bin2bcd(tmp->tm_wday));
   // pcf8563_write (0x05, bin2bcd(tmp->tm_mday));
   // pcf8563_write (0x04, bin2bcd(tmp->tm_hour));
   // pcf8563_write (0x03, bin2bcd(tmp->tm_min ));
   // pcf8563_write (0x02, bin2bcd(tmp->tm_sec ));

   century = ((YEAR_BASE + tmp->tm_year) >= 2000) ? 0x80 : 0;
   buf[0] = bin2bcd(tmp->tm_mday);
   buf[1] = bin2bcd(tmp->tm_wday);
   buf[2] = bin2bcd(tmp->tm_mon) | century;
   buf[3] = bin2bcd(tmp->tm_year % 100);

   pcf8563_writebuf(Days_reg, buf, 4);
}


void pcf8563_reset (void)
{
   //DelayMSec(5);
   /* clear all control & status registers */
   pcf8563_write (Control_status_1_reg, 0x00);
   pcf8563_write (Control_status_2_reg, 0x00);
   pcf8563_write (CLKOUT_control_reg, 0x00);

   /* clear Voltage Low bit */
   pcf8563_write (VL_seconds_reg, pcf8563_read (VL_seconds_reg) & 0x7F);

   /* reset all alarms */
   pcf8563_write (Minute_alarm_reg, 0x00);
   pcf8563_write (Hour_alarm_reg, 0x00);
   pcf8563_write (Day_alarm_reg, 0x00);
   pcf8563_write (Weekday_alarm_reg, 0x00);
}


// set if power fail detected.
int pcf8563_checkClockIntegrity(void)
{
   uchar rd = pcf8563_read(VL_seconds_reg);
   rd &= 0x80;
   rd >>= 7;
   return rd;
}


/* ------------------------------------------------------------------------- */

static uchar pcf8563_read (uchar reg)
{
   unsigned char value = 0;

   I2C_RdReg(PCF8563_ADDR_WRITE, reg, &value, 1);
   return value;
}

static void pcf8563_write (uchar reg, uchar val)
{
   I2C_WrReg(PCF8563_ADDR_WRITE, reg, &val, 1);
}


static int pcf8563_readbuf(uchar reg, uchar *buf, uchar num)
{
   uchar rdnum = num;
   memset(rtcBuffer, 0, sizeof(rtcBuffer));

   if (rdnum > sizeof(rtcBuffer))
      rdnum = sizeof(rtcBuffer);

   I2C_RdReg(PCF8563_ADDR_WRITE, reg, rtcBuffer, rdnum);
   memcpy(buf, rtcBuffer, rdnum);
   return rdnum;
}


static int pcf8563_writebuf(uchar reg, uchar *buf, uchar num)
{
   uchar wrnum = num;

   if (wrnum > 11)  // 0x02 - 0x0C
      wrnum = 11;

   I2C_WrReg(PCF8563_ADDR_WRITE, reg, buf, wrnum);
   return wrnum;
}



static unsigned bcd2bin (uchar n)
{
   return ((((n >> 4) & 0x0F) * 10) + (n & 0x0F));
}

static unsigned char bin2bcd (unsigned int n)
{
   return (((n / 10) << 4) | (n % 10));
}

